//Code taken from
//https://iotdesignpro.com/projects/smart-weighing-machine-for-remote-weight-measurement
//Modified by Pavan Kuchar
//for GoatIT on 15 May 2021

#include <HX711.h>;
#include <WiFi.h>;
#include <WiFiClient.h>;
#include <ThingSpeak.h>;
#include <Wire.h>
#include <LiquidCrystal_I2C.h>

LiquidCrystal_I2C lcd(0x27, 16, 2);
#define LOADCELL_DOUT_PIN  14 //21
#define LOADCELL_SCK_PIN  12 //22

const char* ssid = "Workshop"; //Your Network SSID "488pista";
const char* password = "VIGYAN0030"; //Your Network Password "!@#123wasd";

WiFiClient client;
 
unsigned long myChannelNumber = 1323804; //Your Channel Number (Without Brackets) 858718;
const char * myWriteAPIKey = "8HJ7ZGI6WNBV5FXQ";   //Your Write API Key "NLDSW5N6UHWHSOHO";

HX711 scale;  //(D5, D6) (14, 12);

float calibration_factor = 223; // this calibration factor is adjusted according to my load cell
int units;
float ounces;

void setup() 
{
  Serial.begin(9600);
  Wire.begin(33, 32); //(D2, D1)
  lcd.init(); //lcd.begin()
  lcd.backlight();
  lcd.print("GoatIT"); //"Circuit Digest"
  lcd.setCursor(0,1);
  lcd.print("ESP32 Wg Scale"); //NodeMCU Wg Scale
  
  delay(2000);

  lcd.clear();

  lcd.print("Connecting Wifi");
  lcd.setCursor(0,1);
  lcd.print("And Thingsspeak");
  WiFi.begin(ssid, password);
  ThingSpeak.begin(client);
  
  delay(3000);
  
  Serial.println("HX711 calibration sketch");
  Serial.println("Remove all weight from scale");
  Serial.println("After readings begin, place known weight on scale");
  Serial.println("Press + or a to increase calibration factor");
  Serial.println("Press - or z to decrease calibration factor");

  scale.set_scale();
  scale.tare();  //Reset the scale to 0

  long zero_factor = scale.read_average(); //Get a baseline reading
  Serial.print("Zero factor: "); //This can be used to remove the need to tare the scale. Useful in permanent scale projects.
  Serial.println(zero_factor);
}

void loop() 

{

  scale.set_scale(calibration_factor); //Adjust to this calibration factor

  Serial.print("Reading: ");
  lcd.clear();
    
  units = scale.get_units(), 10;
  if (units < 0)
  {
    units = 0;
  }
 
  if (units == 0)
  {
       lcd.print("Weight :");
       lcd.print(units);
       lcd.print("  g");
       ThingSpeak.writeField(myChannelNumber, 1,units, myWriteAPIKey); //Update in ThingSpeak 
       Serial.print(units);
       Serial.print(" grams"); 
       Serial.print(" calibration_factor: ");
       Serial.print(calibration_factor);
       Serial.println();
       delay(15000);
   } 
  
  if (units > 0)
  {
       lcd.print("Weight :");
       lcd.print(units);
       lcd.print("  g");
       ThingSpeak.writeField(myChannelNumber, 1,units, myWriteAPIKey); //Update in ThingSpeak 
       Serial.print(units);
       Serial.print(" grams"); 
       Serial.print(" calibration_factor: ");
       Serial.print(calibration_factor);
       Serial.println();
       delay(15000);
  } 

 
  if(Serial.available())
  {
    char temp = Serial.read();
    if(temp == '+' || temp == 'a')
      calibration_factor += 1;
    else if(temp == '-' || temp == 'z')
      calibration_factor -= 1;
  }
}
