//Code taken from
//https://iotdesignpro.com/projects/smart-weighing-machine-for-remote-weight-measurement
//Modified by Pavan Kuchar
//for GoatIT on 15 May 2021

#include <HX711.h>;
#include <WiFi.h>;
#include <MFRC522.h>
#include <WiFiClient.h>;
#include <ThingSpeak.h>;
#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include <SPI.h>

#define SS_PIN 21
#define RST_PIN 22
MFRC522 mfrc522(SS_PIN, RST_PIN); 

LiquidCrystal_I2C lcd(0x27, 16, 2);
#define LOADCELL_DOUT_PIN  14 //21
#define LOADCELL_SCK_PIN  12 //22

const char* ssid = "Prabandhan"; //agri Your Network SSID "488pista";
const char* password = "987654321"; //9730005029 Your Network Password "!@#123wasd";

WiFiClient client;
 
unsigned long myChannelNumber = 1323804; //Your Channel Number (Without Brackets) 858718;
const char * myWriteAPIKey = "8HJ7ZGI6WNBV5FXQ";   //Your Write API Key "NLDSW5N6UHWHSOHO";

HX711 scale;  
float calibration_factor = 15050; // 29730 this calibration factor is adjusted according to my load cell
float units;
float units1;
float ounces;

void setup() 
{
  Serial.begin(9600);
  SPI.begin();
  mfrc522.PCD_Init();
  Wire.begin(33, 32); //(D2, D1)
  lcd.init(); //lcd.begin()
  lcd.backlight();
  lcd.print("GoatIT"); //"Circuit Digest"
  lcd.setCursor(0,1);
  lcd.print("ESP32 Wg Scale"); //NodeMCU Wg Scale
  delay(2000);
  lcd.clear();
  lcd.print("Connecting Wifi");
  lcd.setCursor(0,1);
  lcd.print("And Thingsspeak");
  WiFi.begin(ssid, password);
  ThingSpeak.begin(client);
  delay(3000);
  Serial.println("HX711 calibration sketch");
  Serial.println("Remove all weight from scale");
  Serial.println("After readings begin, place known weight on scale");
  Serial.println("Press + or a to increase calibration factor");
  Serial.println("Press - or z to decrease calibration factor");
  scale.begin(LOADCELL_DOUT_PIN, LOADCELL_SCK_PIN);
  scale.set_scale();
  scale.tare();  //Reset the scale to 0
  long zero_factor = scale.read_average(); //Get a baseline reading
  Serial.print("Zero factor: "); //This can be used to remove the need to tare the scale. Useful in permanent scale projects.
  Serial.println(zero_factor);
}

void loop() 

{
   lcd.clear();
   lcd.setCursor(0,0);
   lcd.print("Hello!Scan Card");
  
  if ( ! mfrc522.PICC_IsNewCardPresent()) 
  {
    return;
  }
 
  if ( ! mfrc522.PICC_ReadCardSerial()) 
  {
    return;
  }
  //Show UID on serial monitor
  Serial.print("UID tag :");
  //lcd.print("UID:");
  String content= "";
  byte letter;
  for (byte i = 0; i < mfrc522.uid.size; i++) 
  {
     Serial.print(mfrc522.uid.uidByte[i] < 0x10 ? " 0" : " ");
     Serial.print(mfrc522.uid.uidByte[i], HEX);
     lcd.print(mfrc522.uid.uidByte[i] < 0x10 ? " 0" : " ");
     lcd.print(mfrc522.uid.uidByte[i], HEX);
     content.concat(String(mfrc522.uid.uidByte[i] < 0x10 ? " 0" : " "));
     content.concat(String(mfrc522.uid.uidByte[i], HEX));
  }
  Serial.println();
  Serial.print("Message : ");
  //lcd.print("Message : ");
  content.toUpperCase();
 
  if (content.substring(1) == "6A DA 8F B2")      
  {
    weight();
    delay(1000); 
    ThingSpeak.writeField(myChannelNumber, 1,units1, myWriteAPIKey);
    delay(1000);  
  }
  else if  (content.substring(1) == "2A 98 86 B1")      
  {  
    weight();
    ThingSpeak.writeField(myChannelNumber, 2,units1, myWriteAPIKey);
    delay(1000);  
  }
   else if  (content.substring(1) == "2A 44 92 B2")      
    {
    weight();
    ThingSpeak.writeField(myChannelNumber, 3,units1, myWriteAPIKey);
    delay(1000);  
  }
   else if  (content.substring(1) == "8A EF A6 B1")      
    {
    weight();
    ThingSpeak.writeField(myChannelNumber, 4,units1, myWriteAPIKey);
    delay(1000);  
  }
   else if  (content.substring(1) == "6C D4 F2 2E")      
    {
    weight();
    ThingSpeak.writeField(myChannelNumber, 5,units1, myWriteAPIKey);
    delay(1000);  
  }
   else if  (content.substring(1) == "5A AB AB B1")      
    {
    weight();
    ThingSpeak.writeField(myChannelNumber, 6,units1, myWriteAPIKey);
    delay(1000);  
  }
  else if  (content.substring(1) == "CA D5 A3 B1")      
    {
    weight();
    ThingSpeak.writeField(myChannelNumber, 7,units1, myWriteAPIKey);
    delay(1000);  
  }
  else if  (content.substring(1) == "FA A7 A3 B1")      
    {
    weight();
    ThingSpeak.writeField(myChannelNumber, 8,units1, myWriteAPIKey);
    delay(1000);  
  }
 
  else {
    lcd.setCursor(0,1);
    Serial.println("Access denied");
    lcd.print("Access denied");
    delay(3000);
  }

}

  void weight()  {
  scale.set_scale(calibration_factor); //Adjust to this calibration factor
  Serial.print("Reading: ");
  lcd.clear();
  units = scale.get_units(), 1;
  units1= units*1;
  if (units1 < 0)
  {
    units1 = 0;
  }
 
  if (units1 == 0)
  {
       lcd.print("Weight :");
       lcd.print(units1);
       lcd.print("  kg");
      // ThingSpeak.writeField(myChannelNumber, 2,units1, myWriteAPIKey); //Update in ThingSpeak 
       Serial.print(units1);
       Serial.print(" kgrams"); 
       Serial.print(" calibration_factor: ");
       Serial.print(calibration_factor);
       Serial.println();
       delay(1000);
   } 
  
  if (units1 > 1)
  {
       lcd.print("Weight :");
       lcd.print(units1);
       lcd.print("  kg");
      // ThingSpeak.writeField(myChannelNumber, 1,units1, myWriteAPIKey); //Update in ThingSpeak 
       Serial.print(units1);
       Serial.print(" kilograms"); 
       Serial.print(" calibration_factor: ");
       Serial.print(calibration_factor);
       Serial.println();
       delay(1000);
  } 

 delay(1000);
  if(Serial.available())
  {
    char temp = Serial.read();
    if(temp == '+' || temp == 'a')
      calibration_factor += 1;
    else if(temp == '-' || temp == 'z')
      calibration_factor -= 1;
  }
  }
  
